/**
 * @fileOverview spread sheet api
 * @name api
 * @author Yuhei Aihara <yu.e.yu.4119@gmail.com>
 * https://github.com/iyu/spread2json
 */
import { drive_v3, sheets_v4 } from 'googleapis';
import { Credentials } from 'google-auth-library';
declare class SpreadSheetApi {
    private opts;
    private oAuth2?;
    /**
     * setup
     * @param {Object} options
     * @param {string} options.client_id
     * @param {string} options.client_secret
     * @param {string} [options.redirect_url='http://localhost']
     * @param {Object} [options.token_file]
     * @param {boolean} [options.token_file.use=true]
     * @param {string} [options.token_file.path='./dist/token.json']
     * @example
     * options = {
     *   client_id: 'xxx',
     *   client_secret: 'xxx',
     *   redirect_url: 'http://localhost',
     *   token_file: {
     *     use: true
     *     path: './token.json'
     *   }
     * }
     */
    setup(options: {
        client_id: string;
        client_secret: string;
        redirect_url?: string;
        token_file?: {
            use: boolean;
            path: string;
        };
    }): this;
    /**
     * generate auth url
     * @see https://github.com/google/google-api-nodejs-client/#generating-an-authentication-url
     * @param {Object} [opts={}]
     * @param {string|Array} [opts.scope=SCOPE]
     * @return {string|Error} - URL to consent page.
     */
    generateAuthUrl(opts: {
        scope: string[];
    }): string | Error;
    /**
     * get access token
     * @param {string} code
     */
    getAccessToken(code: string): Promise<Credentials | null | undefined>;
    /**
     * refresh access token
     * @param {Credentials} [credentials] - oAuth2.getToken result
     */
    refreshAccessToken(credentials?: Credentials): Promise<Credentials>;
    /**
     * get spreadsheet
     * @param {Credentials} [credentials] - oAuth2.getToken result
     * @param {Object} [opts] - drive options @see https://developers.google.com/drive/v3/reference/files/list
     * @param {string} [opts.corpus]
     * @param {string} [opts.orderBy]
     * @param {integer} [opts.pageSize]
     * @param {string} [opts.pageToken]
     * @param {string} [opts.q]
     * @param {string} [opts.spaces]
     * @param {Function} callback
     */
    getSpreadsheet(credentials?: Credentials, opts?: drive_v3.Params$Resource$Files$List): Promise<drive_v3.Schema$File[] | undefined>;
    /**
     * get worksheet
     * @param {Credentials} [credentials] - oAuth2.getToken result
     * @param {string} spreadsheetId
     * @param {Function} callback
     */
    getWorksheet(credentials: Credentials | undefined, spreadsheetId: string): Promise<sheets_v4.Schema$Spreadsheet>;
    /**
     * add worksheet
     * @param {Credentials} [credentials] - oAuth2.getToken result
     * @param {string} spreadsheetId
     * @param {string} title - worksheet title
     * @param {number} rowCount
     * @param {number} columnCount
     */
    addWorksheet(credentials: Credentials | undefined, spreadsheetId: string, title: string, rowCount: number, columnCount: number): Promise<sheets_v4.Schema$BatchUpdateSpreadsheetResponse>;
    /**
     * get list raw
     * @param {Credentials} [credentials] - oAuth2.getToken result
     * @param {string} spreadsheetId
     * @param {string} sheetName
     */
    getList(credentials: Credentials | null, spreadsheetId: string, sheetName: string): Promise<sheets_v4.Schema$ValueRange>;
    /**
     * get cells raw
     * @param {Credentials} [credentials] - oAuth2.getToken result
     * @param {string} spreadsheetId
     * @param {string} sheetName
     * @param {Array[]} entry
     */
    batchCells(credentials: Credentials | undefined, spreadsheetId: string, sheetName: string, entry: any[][]): Promise<sheets_v4.Schema$BatchUpdateValuesResponse>;
}
declare const _default: SpreadSheetApi;
export default _default;
