/**
 * @fileOverview spread sheet converter
 * @name index
 * @author Yuhei Aihara <yu.e.yu.4119@gmail.com>
 * https://github.com/iyu/spread2json
 */
import { Credentials } from 'google-auth-library';
import { drive_v3, sheets_v4 } from 'googleapis';
interface Spreadsheet extends drive_v3.Schema$File {
    link: string;
}
interface Worksheet {
    title: string;
    sheetId: string;
    link: string;
}
interface Opts {
    attr_line: number;
    data_line: number;
    ref_keys: string[];
    format: {
        [key: string]: {
            type: string;
            key: string;
            keys: string[];
        };
    };
    key?: string;
    type?: string;
    name?: string;
}
interface JsonSchema {
    type: string;
    items?: JsonSchema;
    properties: {
        [key: string]: JsonSchema;
    };
    allOf: JsonSchema[];
    oneOf: JsonSchema[];
    format?: string;
}
declare type Callback<T> = (err: Error | null, result?: T) => T | undefined;
declare class Spread2Json {
    private opts;
    private _parser;
    private _stringify;
    /**
     * API function
     */
    generateAuthUrl(opts: {
        scope: string[];
    }): string | Error;
    getAccessToken(code: string, callback?: Callback<Credentials | null | undefined>): Promise<Credentials | null | undefined>;
    refreshAccessToken(credentials?: Credentials, callback?: Callback<Credentials>): Promise<Credentials | undefined>;
    /**
     * setup
     * @param {Object} options
     * @param {string} [options.option_cell='A1']
     * @param {number} [options.attr_line=2]
     * @param {number} [options.data_line=4]
     * @param {string} [options.ref_key]
     * @param {string[]} [options.ref_keys=['_id']]
     * @param {Object} options.api
     * @param {string} options.api.client_id
     * @param {string} options.api.client_secret
     * @param {string} [options.api.redirect_url='http://localhost']
     * @param {Object} [options.api.token_file]
     * @param {boolean} [options.api.token_file.use=true]
     * @param {string} [options.api.token_file.path='./dist/token.json']
     * @param {Logger} [logger]
     * @example
     * var options = {
     *     option_cell: 'A1',
     *     attr_line: 2,
     *     data_line: 4,
     *     ref_keys: ['_id'],
     *     api: {
     *         client_id: 'YOUR CLIENT ID HERE',
     *         client_secret: 'YOUR CLIENT SECRET HERE',
     *         redirect_url: 'http://localhost',
     *         token_file: {
     *           use: true
     *           path: './dist/token.json'
     *         }
     *     },
     *     logger: CustomLogger
     * }
     */
    setup(options: {
        option_cell?: string;
        attr_line?: number;
        data_line?: number;
        ref_key?: string;
        ref_keys?: string[];
        api: {
            client_id: string;
            client_secret: string;
            redirect_url?: string;
            token_file?: {
                use: boolean;
                path: string;
            };
        };
        logger: any;
    }): void;
    /**
     * get spreadsheets info
     * @param {Credentials} [credentials] - oAuth2.getToken result
     * @param {Object} [opts] - drive options @see https://developers.google.com/drive/v3/reference/files/list
     * @param {string} [opts.corpus]
     * @param {string} [opts.orderBy]
     * @param {integer} [opts.pageSize]
     * @param {string} [opts.pageToken]
     * @param {string} [opts.q]
     * @param {string} [opts.spaces]
     * @param {Function} [callback]
     * @property {Object[]} response
     * @property {string} response[].kind - 'drive#file'
     * @property {string} response[].id - spreadsheetId
     * @property {string} response[].name - spreadsheet name
     * @property {string} response[].mimeTpe - 'application/vnd.google-apps.spreadsheet'
     * @property {string} response[].link
     */
    getSpreadsheet(): Promise<Spreadsheet[]>;
    getSpreadsheet(credentials: Credentials): Promise<Spreadsheet[]>;
    getSpreadsheet(opts: drive_v3.Params$Resource$Files$List): Promise<Spreadsheet[]>;
    getSpreadsheet(callback: Callback<Spreadsheet[]>): Promise<Spreadsheet[]>;
    getSpreadsheet(credentials: Credentials, opts: drive_v3.Params$Resource$Files$List): Promise<Spreadsheet[]>;
    getSpreadsheet(credentials: Credentials, callback: Callback<Spreadsheet[]>): Promise<Spreadsheet[]>;
    getSpreadsheet(opts: drive_v3.Params$Resource$Files$List, callback: Callback<Spreadsheet[]>): Promise<Spreadsheet[]>;
    getSpreadsheet(credentials: Credentials, opts: drive_v3.Params$Resource$Files$List, callback: Callback<Spreadsheet[]>): Promise<Spreadsheet[]>;
    /**
     * get worksheet info in spreadsheet
     * @param {Credentials} [credentials] - oAuth2.getToken result
     * @param {string} spreadsheetId
     * @param {Function} [callback]
     * @property {Object[]} response
     * @property {string} response[].title
     * @property {integer} response[].sheetId
     * @property {string} response[].link
     * @example
     * > url = 'https://docs.google.com/spreadsheets/d/1YXVzaaxqkPKsr-excIOXScnTQC7y_DKrUKs0ukzSIgo/edit#gid=0'
     * spreadsheetId = '1YXVzaaxqkPKsr-excIOXScnTQC7y_DKrUKs0ukzSIgo'
     * sheetId = 0
     */
    getWorksheet(spreadsheetId: string): Promise<Worksheet[] | undefined>;
    getWorksheet(spreadsheetId: string, callback: Callback<Worksheet[]>): Promise<Worksheet[] | undefined>;
    getWorksheet(credentials: Credentials, spreadsheetId: string): Promise<Worksheet[] | undefined>;
    getWorksheet(credentials: Credentials, spreadsheetId: string, callback: Callback<Worksheet[]>): Promise<Worksheet[] | undefined>;
    /**
     * add worksheet
     * @param {Credentials} [credentials] - oAuth2.getToken result
     * @param {string} spreadsheetId
     * @param {string} [title] - worksheet title
     * @param {number} [rowCount=50]
     * @param {number} [colCount=10]
     * @param {Function} [callback]
     * @property {Object} response
     * @property {string} response.title
     * @property {integer} response.sheetId
     * @property {string} response.link
     * @example
     * > url = 'https://docs.google.com/spreadsheets/d/1YXVzaaxqkPKsr-excIOXScnTQC7y_DKrUKs0ukzSIgo/edit#gid=0'
     * spreadsheetId = '1YXVzaaxqkPKsr-excIOXScnTQC7y_DKrUKs0ukzSIgo'
     */
    addWorksheet(spreadsheetId: string, title?: string, rowCount?: number, colCount?: number, callback?: Callback<Worksheet>): Promise<Worksheet | undefined>;
    addWorksheet(credentials: Credentials, spreadsheetId: string, title?: string, rowCount?: number, colCount?: number, callback?: Callback<Worksheet>): Promise<Worksheet | undefined>;
    /**
     * format
     * @param {Object[]} cells
     * @param {string} cells[].cell
     * @param {string} cells[].column
     * @param {number} cells[].row
     * @param {string} cells[].value
     * @private
     * @example
     * var cells = [
     *     { cell: 'A1', column: 'A', row: 1, value: '{}' },,,,
     * ]
     */
    _format(cells: {
        cell: string;
        column: string;
        row: number;
        value: string;
    }[]): {
        opts: Opts;
        list: {
            [key: string]: any[];
        }[];
    };
    /**
     * find origin data
     * @param {Object} dataMap
     * @param {Object} opts
     * @param {Object} data
     * @private
     */
    _findOrigin(dataMap: {
        [key: string]: any;
    }, opts: {
        key?: string;
        type: string;
        ref_keys: string[];
    }, data: any): any;
    /**
     * get worksheets data
     * @param {Credentials} [credentials] - oAuth2.getToken result
     * @param {string} spreadsheetId
     * @param {string[]} worksheetNames
     * @param {Function} callback
     * @see #getWorksheet arguments.key and returns list.id
     */
    getWorksheetDatas(spreadsheetId: string, worksheetNames: string[]): Promise<any>;
    getWorksheetDatas(spreadsheetId: string, worksheetNames: string[], callback: (err: Error | null, result?: any[], errList?: {
        name: string;
        error: Error;
    }[]) => any): Promise<any>;
    getWorksheetDatas(credentials: Credentials, spreadsheetId: string, worksheetNames: string[]): Promise<any>;
    getWorksheetDatas(credentials: Credentials, spreadsheetId: string, worksheetNames: string[], callback: (err: Error | null, result?: any[], errList?: {
        name: string;
        error: Error;
    }[]) => any): Promise<any>;
    /**
     * sheetDatas to json
     * @param {Object[]} sheetDatas
     * @param {string} sheetDatas.name - work sheet name
     * @param {Object} sheetDatas.opts - sheet option
     * @param {Object[]} sheetDatas.list - sheet data list
     * @param {Function} [callback]
     */
    toJson(sheetDatas: {
        name: string;
        opts: Opts;
        list: any[];
    }[], callback?: (err: any | null, collectionMap: any, optionMap: any) => any): any;
    /**
     * jsonschema to spread2json format
     * @param {Object} schema - jsonschema
     * @param {string[]} refkeys - ref keys
     * @param {string} [path=''] - data path
     */
    convertSchema2Format(schema: JsonSchema, refKeys: string[], path?: string): string[];
    /**
     * multiple sheet
     * @param {string} name
     * @param {string[]} refKeys
     * @param {string[]} format
     * @param {Object[]} datas
     */
    splitSheetData(name: string, refKeys: string[], format: string[], datas: any[]): {
        name: string;
        opts: {
            name: string;
            ref_keys: string[];
            attr_line: number;
            desc_line: number;
        };
        format: string[];
        description: string[];
        datas: any[];
    }[];
    /**
     * sheetDatas to cells
     * FIXME array of array is unsupported
     * @param {Object} worksheetData
     * @param {Object} [worksheetData.opts={}] - sheet options
     * @param {string[]} worksheetData.format - data attribute
     * @param {string[]} worksheetData.description - attribute description
     * @param {Object[]} worksheetData.datas - datas
     * @param {string} [worksheetData.name] - sheet name
     */
    toCells(worksheetData: {
        opts?: any;
        format: string[];
        description: string[];
        datas: any[];
        name?: string;
    }): {
        maxCol: number;
        maxRow: number;
        cells: {
            col: number;
            row: number;
            value: string;
        }[];
        name: string | undefined;
        opts: any;
    };
    /**
     * update worksheets data
     * @param {Credentials} [credentials] - oAuth2.getToken result
     * @param {string} spreadsheetId
     * @param {string} worksheetName
     * @param {Object[]} cells
     * @param {number} cells.col
     * @param {number} cells.row
     * @param {string} cells.value
     * @param {Function} callback
     * @see #getWorksheet arguments.key and returns list.id
     */
    updateWorksheetDatas(spreadsheetId: string, worksheetName: string, cells: {
        col: number;
        row: number;
        value: string;
    }[]): Promise<sheets_v4.Schema$BatchUpdateValuesResponse>;
    updateWorksheetDatas(spreadsheetId: string, worksheetName: string, cells: {
        col: number;
        row: number;
        value: string;
    }[], callback: Callback<sheets_v4.Schema$BatchUpdateValuesResponse>): Promise<sheets_v4.Schema$BatchUpdateValuesResponse>;
    updateWorksheetDatas(credentials: Credentials, spreadsheetId: string, worksheetName: string, cells: {
        col: number;
        row: number;
        value: string;
    }[]): Promise<sheets_v4.Schema$BatchUpdateValuesResponse>;
    updateWorksheetDatas(credentials: Credentials, spreadsheetId: string, worksheetName: string, cells: {
        col: number;
        row: number;
        value: string;
    }[], callback: Callback<sheets_v4.Schema$BatchUpdateValuesResponse>): Promise<sheets_v4.Schema$BatchUpdateValuesResponse>;
    static readonly COLUMN_KEYMAP: string[];
}
declare const _default: Spread2Json;
export default _default;
